﻿using Android.App;
using Android.Graphics;
using Android.OS;
using Android.Views;
using System;
using System.IO;
using ThinkGeo.MapSuite.AndroidEdition;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Portable;

namespace SingTileRefreshTest
{
    [Activity(MainLauncher = true, Label = "ArpentGIS", ConfigurationChanges = Android.Content.PM.ConfigChanges.Orientation | Android.Content.PM.ConfigChanges.ScreenSize)]
    public class MainActivity : Activity
    {
        private MapView mapView;
        
      private LayerOverlay layerOverlay;
        private InMemoryFeatureLayer vehicleLayer;
      
      LayerOverlay bkgOverlay;
      OpenStreetMapTileOverlay osmTileOverlay;

        static MainActivity()
        {
            Pcl.Current = new AndroidEditionPclShareEnviroment();
        }

        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);
            SetContentView(Resource.Layout.Main);

            mapView = FindViewById<MapView>(Resource.Id.androidMap);
            mapView.MapUnit = GeographyUnit.Meter;
            mapView.ZoomLevelSet = new GoogleMapsZoomLevelSet();

            foreach (MapTool m in mapView.MapTools)
              m.IsEnabled = false;

            bkgOverlay = new LayerOverlay();
            bkgOverlay.TileType = TileType.SingleTile;
            mapView.Overlays.Add(bkgOverlay);

            //mapView.Refresh();


            osmTileOverlay = new OpenStreetMapTileOverlay();
            mapView.Overlays.Add(osmTileOverlay);

            mapView.CurrentExtent = osmTileOverlay.GetBoundingBox();

          
          /*  InMemoryFeatureLayer m_GpsCurrentPos = new InMemoryFeatureLayer();
       //     m_GpsCurrentPos.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(GetGeoImageFromImageId(Resource.Drawable.Micheal_vehicle));
       //     m_GpsCurrentPos.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
          //  m_GpsCurrentPos.IsVisible = false;

            LayerOverlay gpsOverlay = new LayerOverlay();
            gpsOverlay.Layers.Add(m_GpsCurrentPos);
            gpsOverlay.TileType = TileType.SingleTile;
            mapView.Overlays.Add(gpsOverlay);
          */
            layerOverlay = new LayerOverlay();
            layerOverlay.TileType = TileType.SingleTile;
          
            vehicleLayer = new InMemoryFeatureLayer();
            vehicleLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(GetGeoImageFromImageId(Resource.Drawable.Micheal_vehicle));
            vehicleLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(vehicleLayer);
            mapView.Overlays.Add(layerOverlay);

            System.Timers.Timer timer = new System.Timers.Timer();
            timer.Interval = 1000;
            timer.Elapsed += timer_Elapsed;
            timer.Start();
        }

        protected override void OnStart()
        {
          base.OnStart();
          ActionBar.SetDisplayHomeAsUpEnabled(true);
        }

        public override bool OnCreateOptionsMenu(IMenu menu)
        {
          base.OnCreateOptionsMenu(menu);

          MenuInflater inflater = this.MenuInflater;

          inflater.Inflate(Resource.Menu.menu, menu);

          return true;
        }

        int step = 1;
        public override bool OnOptionsItemSelected(IMenuItem item)
        {
          if (step == 1)
          {
            // remove OSM
            mapView.Overlays.Remove(osmTileOverlay);

            // add a cached tiff
            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();
            bitmapTileCache.CacheDirectory = "/mnt/sdcard/ArpentGIS";
            bitmapTileCache.CacheId = "BkgdCachedTiles";
            bitmapTileCache.TileAccessMode = TileAccessMode.ReadAddDelete;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;
            bitmapTileCache.ClearCache();
            bkgOverlay.TileCache = bitmapTileCache;
            // bkgOverlay.TileSnappingMode = TileSnappingMode.Snapping;  ???
            bkgOverlay.TileType = TileType.MultiTile;

            GeoTiffRasterLayer geoTiffRasterLayer = new GeoTiffRasterLayer("/mnt/sdcard/ArpentGIS/savipol_1.tif");
            geoTiffRasterLayer.UpperThreshold = double.MaxValue;
            geoTiffRasterLayer.IsGrayscale = false;
            geoTiffRasterLayer.LowerThreshold = 0;
            geoTiffRasterLayer.Open();

            bkgOverlay.Layers.Add(geoTiffRasterLayer);

            mapView.CurrentExtent = geoTiffRasterLayer.GetBoundingBox();
            mapView.Refresh();

            step++;
          }

          else
          {
            // Remove the tiff
            bkgOverlay.Layers.Clear();
            bkgOverlay.TileType = TileType.SingleTile;
            bkgOverlay.TileCache.ClearCache();
            //bkgOverlay.TileCache = null;

            // add a shape

              ShapeFileFeatureLayer shapeRasterLayer = new ShapeFileFeatureLayer("/mnt/sdcard/ArpentGIS/DPT06-L93.shp");

              GeoColor color = GeoColor.GetRandomGeoColor(RandomColorType.All);

              AreaStyle styleA = new AreaStyle();
              styleA.FillSolidBrush = new GeoSolidBrush(color);
              styleA.OutlinePen = new GeoPen(GeoColor.SimpleColors.Black, 1);

              LineStyle styleL = new LineStyle();
              styleL.OuterPen = new GeoPen(GeoColor.SimpleColors.Black, 1);

              PointStyle styleP = new PointStyle();
              styleP.PointType = PointType.Symbol;
              styleP.SymbolType = PointSymbolType.Circle;
              styleP.SymbolSolidBrush = new GeoSolidBrush(color);
              styleP.SymbolPen = new GeoPen(GeoColor.SimpleColors.Black, 1);
              styleP.SymbolSize = 10;

              shapeRasterLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(styleA);
              shapeRasterLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(styleL);
              shapeRasterLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(styleP);
              shapeRasterLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

              shapeRasterLayer.Open();

              bkgOverlay.Layers.Add(shapeRasterLayer);

              mapView.CurrentExtent = shapeRasterLayer.GetBoundingBox();
              mapView.Refresh();
          }

          return true;
        }

        private void timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
          mapView.Post(() =>
                       {
                         lock (vehicleLayer)
                         {
                           vehicleLayer.InternalFeatures.Clear();
                           vehicleLayer.InternalFeatures.Add(new Feature(GetRandomPointShape()));
                           layerOverlay.Refresh();
                         }
                       });
        }

        private PointShape GetRandomPointShape()
        {
            Random raX = new Random();
            double scaleX = raX.NextDouble();

            Random raY = new Random((int)(scaleX * 100));
            double scaleY = raY.NextDouble();

            double valuex = mapView.CurrentExtent.LowerRightPoint.X - mapView.CurrentExtent.UpperLeftPoint.X;
            double valueY = mapView.CurrentExtent.LowerRightPoint.Y - mapView.CurrentExtent.UpperLeftPoint.Y;

            return new PointShape(mapView.CurrentExtent.UpperLeftPoint.X + valuex * scaleX, mapView.CurrentExtent.UpperLeftPoint.Y + valueY * scaleY);
        }

        private GeoImage GetGeoImageFromImageId(int imageId)
        {
            MemoryStream ms = new MemoryStream();
            BitmapFactory.DecodeResource(this.Resources, imageId).Compress(Bitmap.CompressFormat.Png, 100, ms);

            return new GeoImage(ms);
        }
    }
}

